/** 
 * @file Gainer.h
 * @brief Gainer Class Header
 * @author dandelion
 * @date 2009-04-18
 * @version $Id: Gainer.h, v0.1$
 * 
 * Copyright (C) 2009 dandelion. All rights reserved.
 */


#ifndef _GAINER_H_
#define _GAINER_H_

#include <string>
#include <vector>
#include <process.h>
#include <windows.h>

class Gainer;

namespace GainerInternal
{
	class Exception
	{

	}; // Exception
} // GainerInternal


// COM port number==================
enum COM_PORT
{
	COM1 = 1,
	COM2,
	COM3,
	COM4,
	COM5,
	COM6,
	COM7,
	COM8,
	COM9,
	COM10
};
//==================================

// Configulation Mode ==============
enum MODE
{
	MODE1 = 1,
	MODE2,
	MODE3,
	MODE4,
	MODE5,
	MODE6,
	MODE7
};
//==================================

#define SEND_TIME 50

/**
 * Gainer class. 
 * @author dandelion
 * @version 0.1, 2009/04/18
 */
class Gainer
{
public:
   /**
    * Event handler calback
    */
	typedef void (*callback_t)(void);

   /**
    * Gainer class default constructor.
    * @param portNum COM port number
    * @param mode configutation mode (1-7)
    * @param verbose verbose output
    */
	Gainer(int portNum=1, int mode=1, bool verbose=false);

   /**
    * Gainer class destructor.
    */
	~Gainer();

   /**
    * Turn on the LED on gainer. command "h*" 
    * @return
    */
	void turnOnLED();
   
   /**
    * Turn off the LED on gainer. command "l*" 
    * @return
    */
	void turnOffLED();

// Not implemented yet. (2009/04/18) ------
#if 0
	void set_matrix(ary);
	void set_analog_output(int n);

	void continuous_digital_inputs();
	void continuous_analog_inputs();
	void exit_continuos();
#endif
//-----------------------------------------

   /**
    * It can bringing values to digitalIn[] at one time.
    * @return
    */
	void peekDigitalInput();

   /**
    * It can bringing values to analogIn[] at one time.
    * @return
    */
	void peekAnalogInput();

   /**
    * It makes 0V or +5V on digital output port.
    * @param value values of the digital output ports true:1 false:0
    * @return
    */
	void digitalOutput(int value);

   /**
    * It makes +5V on digital output port.
    * @param port number of the digital output port
    * @return
    */
	void setHigh(int port);

#if 0
   /**
    * It makes +5V on digital output port.
    * @param portArray number of the digital output port
    * @return
    */
	void setHigh(int *portArray);
#endif

   /**
    * It makes 0V on digital output port.
    * @param port number of the digital output port
    * @return
    */
	void setLow(int port);

#if 0
   /**
    * It makes 0V on digital output port.
    * @param portArray number of the digital output port
    * @return
    */
	void setLow(int *portArray);
#endif

   /**
    * set button pressed event Callback function
    * @param funcp button pressed event Callback function pointer
    * @return
    */
	void setOnPressedCallback(callback_t funcp){on_pressed = funcp;}

   /**
    * set button released event Callback function
    * @param funcp button released event Callback function pointer
    * @return
    */
	void setOnReleasedCallback(callback_t funcp){on_released = funcp;}

private:
   /**
    * setup COM port.
    * @return 0: normal termination, 1: error, 2: error
    */
	int setupCOMPort();

   /**
    * Software reset. User should not use this.
    * @return
    */
	void reboot();

   /**
    * set configuration mode.
    * @param mode configutation mode (1-7)
    * @return
    */
	void setConfiguration(int mode);

   /**
    * 
    * @param cmd command string
    * @param wait wait time(ms)
    * @return
    */
	void command(const std::string &cmd, int wait=0);

	void command_send(const std::string &cmd);
	void processEvent(std::string &event);

	// properties
	bool led_;
	std::vector<int> analogInputs;
	std::vector<bool> digitalInputs;

   /**
    * configuration
    */
	int config_;

   /**
    * COM port number
    */
	int portNum_;
	bool endFlag;

   /**
    * callback function pointer
    */
	callback_t on_pressed, on_released;

   /**
    * handle for receiving thread
    */
	HANDLE handle;

	enum pin_t {AIN = 0, DIN, AOUT, DOUT};
	static const int CONFIG[][4];
	static const int MATRIX_LED_CONFIGURATION = 7;
	static unsigned __stdcall receiver(void *arg);
};

#endif // __GAINER_H_
